float4x4 gWorld : WORLD;
float4x4 gWorldViewProj : WORLDVIEWPROJECTION; 
float3 gLightDirection = float3(-0.577f, -0.577f, 0.577f);

float4x4 gBones[70];

Texture2D gDiffuseMap;
Texture2D gOpacityMap;

bool gIgnoreLightDirection;

SamplerState samLinear
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Wrap;// or Mirror or Clamp or Border
    AddressV = Wrap;// or Mirror or Clamp or Border
};

RasterizerState Solid
{
	FillMode = SOLID;
	CullMode = FRONT;
};

struct VS_INPUT{
	float3 pos : POSITION;
	float3 normal : NORMAL;
	float2 texCoord : TEXCOORD;
    float4 boneIndices : BLENDINDICES;
    float4 boneWeights : BLENDWEIGHTS;
};
struct VS_OUTPUT{
	float4 pos : SV_POSITION;
	float3 normal : NORMAL;
	float2 texCoord : TEXCOORD;
};

DepthStencilState EnableDepth
{
	DepthEnable = TRUE;
	DepthWriteMask = ALL;
};

RasterizerState NoCulling
{
	CullMode = NONE;
};

BlendState NoBlending
{
	BlendEnable[0] = FALSE;
};

BlendState EnableBlending
{
    BlendEnable[0] = TRUE;
    DestBlend = INV_SRC_ALPHA;
    SrcBlend = SRC_ALPHA;
};

//--------------------------------------------------------------------------------------
// Vertex Shader
//--------------------------------------------------------------------------------------
VS_OUTPUT VS(VS_INPUT input)
{
    VS_OUTPUT output;
    float4 transformedPosition = float4(0, 0, 0, 0);
    float3 transformedNormal = float3(0, 0, 0);

	// For each bone
    for (int i = 0; i < 4; ++i)
    {
        if (input.boneIndices[i] >= 0)
        {
            int boneIndex = input.boneIndices[i];
            float boneWeight = input.boneWeights[i];

			// Calculate BoneSpace Position & Normal
            float4 boneSpacePos = mul(float4(input.pos, 1.0f), gBones[boneIndex]);
            float3 boneSpaceNormal = mul(input.normal, (float3x3) gBones[boneIndex]);

			// Append this Position & Normal to the transformedPosition & transformedNormal
            transformedPosition += boneSpacePos * boneWeight;
            transformedNormal += boneSpaceNormal * boneWeight;
        }
    }

	// Ensure that the w-component of the transformedPosition is 1
    transformedPosition.w = 1.0f;

	// Step 1:	Transform the transformedPosition to Clipping Space (WVP Matrix)
    output.pos = mul(transformedPosition, gWorldViewProj);
	
	// Step 2:	Transform the transformedNormal to World Space (World Matrix)  (Rotation only!)
    output.normal = normalize(mul(transformedNormal, (float3x3) gWorld));

    output.texCoord = input.texCoord;

    return output;
}

//--------------------------------------------------------------------------------------
// Pixel Shader
//--------------------------------------------------------------------------------------
float4 PS(VS_OUTPUT input) : SV_TARGET
{
    float opacity = gOpacityMap.Sample(samLinear, input.texCoord).r;
    clip(opacity - 0.1f);
    
    float4 diffuseColor = gDiffuseMap.Sample(samLinear, input.texCoord);
    float3 color_rgb = diffuseColor.rgb;
	//float color_a = diffuseColor.a;
	
	//HalfLambert Diffuse :)
    float diffuseStrength = .8;
    if (!gIgnoreLightDirection)
    {
        diffuseStrength = dot(input.normal, -gLightDirection);
        diffuseStrength = diffuseStrength * 0.5 + 0.5;
        diffuseStrength = saturate(diffuseStrength);
    }
    
    color_rgb = color_rgb * diffuseStrength;
    return float4(color_rgb, opacity);
}

//--------------------------------------------------------------------------------------
// Technique
//--------------------------------------------------------------------------------------
technique11 Default
{
    pass P0
    {
		SetRasterizerState(NoCulling);
		SetDepthStencilState(EnableDepth, 0);
        SetBlendState(EnableBlending, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);

		SetVertexShader( CompileShader( vs_4_0, VS() ) );
		SetGeometryShader( NULL );
		SetPixelShader( CompileShader( ps_4_0, PS() ) );
    }
}

